# SPDX-License-Identifier: Apache-2.0

import os
import json
import random
import re
from datetime import timedelta
from redis import Redis
from rq import Queue
from rq_scheduler import Scheduler
from vat_evidence_collector.vat_client import VatClient
from clouditor_interface.assessment_interface import AssessmentInterface
from clouditor_interface.assessment_interface_messages import create_resource, create_assess_evidence_request, print_evidence
from clouditor_interface.clouditor_authentication import ClouditorAuthentication
from clouditor_interface.ids_provider import get_tool_id, get_cloud_service_id
import uuid
from log_conf.log_conf import LOGGER

REDIS_HOST = os.environ.get("redis_host")
REDIS_PORT = int(os.environ.get("redis_port"))
REDIS_QUEUE = os.environ.get("redis_queue")

VAT_PROTOCOL = os.environ.get("vat_protocol")
VAT_HOST = os.environ.get("vat_host")
VAT_PORT = int(os.environ.get("vat_port"))
VAT_API_PREFIX = os.environ.get("vat_api_prefix")
VAT_CHECK_HOSTS = (os.environ.get("vat_check_hosts")).split(',')
VAT_NMAP_CHECK_TIMEOUT = int(os.environ.get("vat_nmap_check_timeout"))
VAT_W3AF_CHECK_TIMEOUT = int(os.environ.get("vat_w3af_check_timeout"))

VAT_VULNERABILITY_LEVEL = int(os.environ.get("vat_vulnerability_level"))

DUMMY_VAT = os.environ.get("dummy_vat").lower() in ('true', '1', 't')

vat = VatClient(VAT_PROTOCOL, VAT_HOST, VAT_PORT, VAT_API_PREFIX, LOGGER)

oauth_client = ClouditorAuthentication(LOGGER)

ai = AssessmentInterface(LOGGER)

redis = Redis(REDIS_HOST, REDIS_PORT)
queue = Queue(REDIS_QUEUE, connection=redis)
scheduler = Scheduler(connection=redis, queue=queue)

def create_new_nmap_scan():
    if not VAT_CHECK_HOSTS:
        LOGGER.debug("No hosts to check with VAT")
        return None
    
    with open('./vat_evidence_collector/vat_nmap_request_config.json', 'r') as f:
        file_data = json.load(f)

    for vat_check_host in VAT_CHECK_HOSTS:      
        file_data["target"]["ip"] = str(vat_check_host)

        headers = {}
        payload={'name': 'nmap scan: [' + file_data["target"]["ip"] + ']',
                'indicator_id': '',
                'type': 'vuln_scan',
                'tool': 'null',
                'timeout': str(VAT_NMAP_CHECK_TIMEOUT) + 'm',
                'files': ('config.json', json.dumps(file_data), 'application/json')}

        response = vat.req('POST', 'tasks/', headers=headers, fields=payload)

        new_scan_id = response["id"]

        new_job = scheduler.enqueue_in(timedelta(minutes=VAT_NMAP_CHECK_TIMEOUT), run_nmap_collector, new_scan_id)

        LOGGER.debug("New VAT Nmap scan analysis successfully scheduled: " + str(new_job))

def create_new_w3af_scan(ports):
    with open('./vat_evidence_collector/vat_w3af_request_config.json', 'r') as f:
        file_data = json.load(f)

    for vat_check_host in VAT_CHECK_HOSTS:
        host_scans = []

        for port in ports:
            for prefix in ["http", "https"]:
                file_data["target"]["url"] = prefix + "://" + vat_check_host + ":" + str(port)

                headers = {}
                # Payload name formed as this due to VAT bug causing checks to fail if name includes URL
                payload={'name': 'w3af scan: [' + prefix + ',' + vat_check_host + ',' + str(port) + ']',
                        'indicator_id': '',
                        'type': 'vuln_scan',
                        'tool': 'null',
                        'timeout': str(VAT_W3AF_CHECK_TIMEOUT) + 'm',
                        'files': ('config.json', json.dumps(file_data), 'application/json')}

                response = vat.req('POST', 'tasks/', headers=headers, fields=payload)
                
                new_scan_id = response["id"]

                host_scans.append(new_scan_id)
                
                LOGGER.debug("New VAT w3af scan successfully requested for " + str(file_data["target"]["url"]))

        new_job = scheduler.enqueue_in(timedelta(minutes=VAT_W3AF_CHECK_TIMEOUT), run_w3af_collector_main, host_scans)

        LOGGER.debug("New VAT w3af scan analysis successfully scheduled: " + str(new_job))

def generate_evidence(raw_evidence, scanned_host, number_of_vulnerabilities_found):
    vulnerability_discovery = { "numberOfVulnerabilitiesFound":  number_of_vulnerabilities_found }

    resource = create_resource(scanned_host, None, vulnerability_discovery)

    return create_assess_evidence_request(uuid.uuid1(), get_cloud_service_id("vat"), get_tool_id(), raw_evidence, resource)

def get_scan_details(vat, scan_id):
        headers = {}
        headers['Content-Type'] = 'application/json'

        body = vat.req('GET', 'tasks/' + scan_id + '/executions', headers=headers)    

        try:
            if body['items'][0]['result']['message'] == "Container finished execution":
                return body
        except Exception:
            LOGGER.exception("VAT scan " + scan_id + " details not available: scan failed or timed out")
            raise

def run_nmap_collector(scan_id):
    LOGGER.debug("Executing scheduled VAT Nmap scan analysis job: " + scan_id)
    
    raw_evidence = get_scan_details(vat, scan_id)

    scan_output = json.loads(raw_evidence['items'][0]['result']['output'])

    ports = []

    for report in scan_output['reports']:
        ports.append(int(re.findall(r'\b\d+\b', report['short_desc'])[0]))

    if len(ports) == 0:
        scanned_host = scan_output['reports'][0]['target']
        LOGGER.debug("Scanned host " + str(scanned_host) + " has no open ports.")

        evidence = generate_evidence(raw_evidence, scanned_host, 0)

        ai.send_evidence(evidence, oauth_client.get_token())
        print_evidence(LOGGER, evidence.evidence)
    else:
        create_new_w3af_scan(ports)

def run_w3af_collector(scans):    
    LOGGER.debug("Executing scheduled VAT w3af scans analysis job")
    
    raw_evidences = []
    scanned_host = None
    number_of_vulnerabilities_found = 0
    
    for scan in scans:
        LOGGER.debug("Analysing w3af scan: " + scan)

        try:
            raw_evidence = get_scan_details(vat, scan)
        except Exception:
            pass
            continue

        raw_evidences.append(raw_evidence)

        scan_output = json.loads(raw_evidence['items'][0]['result']['output'])
        
        if len(scan_output['reports']) <= 0:
            continue
        
        scanned_host = scan_output['reports'][0]['target']

        for report in scan_output['reports']:
            if int(report['w_risk_level']) >= VAT_VULNERABILITY_LEVEL:
                number_of_vulnerabilities_found += 1

        LOGGER.debug("Number of vulnerabilities (above severity level " + str(VAT_VULNERABILITY_LEVEL) + ") found for : " + str(number_of_vulnerabilities_found))

    evidence = generate_evidence(raw_evidences, scanned_host, number_of_vulnerabilities_found)

    ai.send_evidence(evidence, oauth_client.get_token())
    print_evidence(LOGGER, evidence.evidence)

def run_demo_collector():
    LOGGER.debug("Simulating VAT analysis job")

    evidence = generate_evidence("", "dummyVAT", random.randint(0,20))

    ai.send_evidence(evidence, oauth_client.get_token())
    print_evidence(LOGGER, evidence.evidence)

def run_w3af_collector_main(scan_id):
    try:
        run_w3af_collector(scan_id)
    except Exception as err:
        LOGGER.exception("Exception caught in VAT Evidence Collector run_w3af_collector(): " + str(err), exc_info=False)

def main():
    if DUMMY_VAT:
        try:
            run_demo_collector()
        except Exception as err:
            LOGGER.exception("Exception caught in VAT Evidence Collector run_demo_collector(): " + str(err), exc_info=False)
    else:
        try:
            create_new_nmap_scan()
        except Exception as err:
            LOGGER.exception("Exception caught in VAT Evidence Collector create_new_nmap_scan(): " + str(err), exc_info=False)

if __name__ == "__main__":
    main()
