# SPDX-License-Identifier: Apache-2.0

import os
from wazuh_evidence_collector.wazuh_client import WazuhClient
from elasticsearch import Elasticsearch
from clouditor_interface.assessment_interface import AssessmentInterface
from clouditor_interface.assessment_interface_messages import create_resource, create_assess_evidence_request, print_evidence
from clouditor_interface.clouditor_authentication import ClouditorAuthentication
from clouditor_interface.ids_provider import get_tool_id, get_cloud_service_id
from wazuh_evidence_collector.checker import Checker
from wazuh_evidence_collector.demo_checker import DemoChecker
import uuid
from log_conf.log_conf import LOGGER

DUMMY_WAZUH_MANAGER = os.environ.get("dummy_wazuh_manager").lower() in ('true', '1', 't')

WAZUH_HOST = os.environ.get("wazuh_host")
WAZUH_PORT = int(os.environ.get("wazuh_port"))
WAZUH_USERNAME = os.environ.get("wazuh_username")
WAZUH_PASSWORD = os.environ.get("wazuh_password")

ELASTIC_HOST = os.environ.get("elastic_host")
ELASTIC_PORT = int(os.environ.get("elastic_port"))
ELASTIC_USERNAME = os.environ.get("elastic_username")
ELASTIC_PASSWORD = os.environ.get("elastic_password")

if not DUMMY_WAZUH_MANAGER:
    wc = WazuhClient(WAZUH_HOST, WAZUH_PORT, WAZUH_USERNAME, WAZUH_PASSWORD, LOGGER)

    es = Elasticsearch(
            ELASTIC_HOST,
            http_auth=(ELASTIC_USERNAME, ELASTIC_PASSWORD),
            scheme='https',
            port=ELASTIC_PORT,
            use_ssl=False,
            verify_certs=False,
            ssl_show_warn=False,
        )

oauth_client = ClouditorAuthentication(LOGGER)

ai = AssessmentInterface(LOGGER)

def main():
    try:
        run_collector()
    except Exception as err:
        LOGGER.exception("Exception caught in Wazuh Evidence Collector run_collector(): " + str(err), exc_info=False)

# Wrapper function that runs all the checks (for every manager/agent)
def run_collector():
    if DUMMY_WAZUH_MANAGER is False:
        LOGGER.debug("Using Wazuh Evidence Collector Checker to evaluate Wazuh logs")
        checker = Checker(wc, es, LOGGER)
    else:
        LOGGER.debug("Simulating evidence generation using Wazuh Evidence Collector Demo Checker")
        checker = DemoChecker()

    # Get list of all agent ids (including manager's)
    def get_agents(wc):
        body = wc.req('GET', 'agents')

        agent_list = []
        try:
            for agent in body['data']['affected_items']:
                agent_list.append([agent['id'], agent['name']])
        except TypeError:
            LOGGER.exception("Invalid agent list", exc_info=False)
            raise

        return body, agent_list

    body, agent_list = get_agents(wc) if not DUMMY_WAZUH_MANAGER else ({}, [[0, "dummyAgent0"], [1, "dummyAgent1"]])

    ae_req_list = []

    LOGGER.debug("Generating evidences for " + str(len(agent_list)) + " agents...")
    for agent in agent_list:
        ae_req_list.append(generate_evidence(agent, checker))

    LOGGER.debug("Forwarding " + str(len(ae_req_list)) + " evidences...")
    for ae_req in ae_req_list:
        ai.send_evidence(ae_req, oauth_client.get_token())
        print_evidence(LOGGER, ae_req.evidence)

    return ae_req_list

# Run checks and generate evidence
def generate_evidence(agent, checker):
    raw_evidence = []

    # MalwareProtectionEnabled
    evidence, result_syscheck = checker.check_syscheck(agent)
    raw_evidence.append(evidence)

    evidence, result_rootcheck = checker.check_rootcheck(agent)
    raw_evidence.append(evidence)

    evidence, result_virus_total = checker.check_virus_total_integration()
    raw_evidence.append(evidence)

    evidence, result_clamd_process = checker.check_clamd_process(agent)
    raw_evidence.append(evidence)

    evidence, result_clamd_logs = checker.check_clamd_logs_elastic(agent)
    raw_evidence.append(evidence)

    security_events_count = checker.check_security_events(agent)

    if result_syscheck and result_rootcheck and \
        (result_virus_total or (result_clamd_process and result_clamd_logs)):
        malware_protection = { "malwareProtection": { "enabled": True }}
    else: 
        malware_protection = { "malwareProtection": { "enabled": False }}

    # TODO: implement metrics
    malware_protection["malwareProtection"].update({ "daysSinceActive": None, "numberOfThreatsFound": security_events_count})

    # MalwareProtectionOutput
    evidence, result_alert_integration = checker.check_alert_integrations()
    raw_evidence.append(evidence)

    if result_alert_integration:
        malware_protection["malwareProtection"].update({ "applicationLogging": { "enabled": True, "loggingService": ["?"], "retentionPeriod": None }})
    else: 
        malware_protection["malwareProtection"].update({ "applicationLogging": { "enabled": False, "loggingService": [], "retentionPeriod": None }})

    resource = create_resource(agent[1], None, malware_protection)
    return create_assess_evidence_request(uuid.uuid1(), get_cloud_service_id("wazuh"), get_tool_id(), raw_evidence, resource)

if __name__ == "__main__":
    main()
